function [design,value,model,runtime,failedCount]=MAMSD(num_factors,runs,n2fi,n3fi,n4fi,ndif,numModels,A,b,Aeq,beq,num_tests,tolerance)
%Generates a model robust D optimal mixture design. 

%Inputs: num_factors is the number of components in the experiment. Runs is the number of runs
%desired. n2fi is the number of second order terms xi*xj in the model. n3fi
%is the number of third order terms xi*xj*xk. ndif is the number of
%difference terms of the form xi*xj*(xi-xj). numModels is the number of
%models sampled for the approximate model space. A is a matrix containing
%the linear inequality constraints Ax<=b; b is its associated vector of
%bounds. Aeq is the matrix of equality constraints (ie, the mixture
%constraint) and beq is its associated vector of constants (ie 1).
%num_tests is the number of times the algorithm will run with a new random
%restart. 

%****Note: A must contain the upper and lower bound constraints in its
%first 2*num_factors rows. The lower bound for each factor must be
%immediately before the same factor's upper bound. The multicomponent
%constraints must be placed in the last rows after the lower/upper bound
%constraints. These can occur in any order. 


%Outputs: design is the best design found. Value is
%-log(det(X'X)) where X is design expanded according to model. If
%numModels is greater than 1, then value is the mean of the above across
%all models selected. Model contains the models selected for the
%approximate model space. 



tic;
options=optimset('Algorithm','active-set','Display','off','GradObj','on','TolCon',tolerance,'TolFun',tolerance,'TolX',tolerance);%,'DerivativeCheck','on');
warning('off','MATLAB:nearlySingularMatrix');
    model=select_models(num_factors,n2fi,n3fi,n4fi,ndif,numModels);
    %list will contain the final design from each algorithm try and
    %list_values will contain the associated value.
    list=zeros(runs,num_factors,num_tests);
    list_values=zeros(1,num_tests);
    
    parfor (test=1:num_tests,4)
    initial=rejection_sampling(runs,num_factors,A,b); 
    settings=initial;
    previous=calc_obj(model,settings,ndif);
    
    %Main loop for each algorithm try. Loops through each row and fmincon
    %finds the greatest improvement for that row to the objective value.
    %Then checks for overall improvement to the objective value after all
    %rows have been improved. Loop continues until no improvement is found.
   
    while (true)
        try
            for row=1:runs
               current_row=settings(row,:);
               objective=@(x)mixture_objective(x,model,settings,row,ndif);
               new_row=fmincon(objective,current_row,A,b,Aeq,beq,zeros(1,length(current_row)),[],[],options);
               settings(row,:)=new_row;
            end
            current=calc_obj(model,settings,ndif);

             %Check for improvement.
            if current<previous           
                previous=current;
            else
                break
            end
        catch
            %Error: probably nan or inf in svd call inside of fmincon
            %Ignore this test by setting value to inf
            current = inf;
            break
        end
  
    end
        list(:,:,test)=settings;
        list_values(test)=current; 
    end
     %Take the design with the best objective value.
    [value,index]=min(list_values);
    value=exp(-value);
    design=list(:,:,index);
    runtime=toc;
    failedCount = sum(list_values==inf);

end

function obj=calc_obj(model,settings,ndif)
numModel=length(model(1,1,:));
objective=zeros(1,numModel);
for i=1:numModel
   D=expandDesign(settings,model(:,:,i),ndif);
   objective(i) = prod(svd(D));
end
obj=-log(mean(objective));
end

function [objective,gradient]=mixture_objective(current_row,model,settings,row,ndif)
    settings(row,:)=current_row;
    numModel=length(model(1,1,:));
    objective=zeros(1,numModel);
    
    num_cont=length(current_row);
    numModel=length(model(1,1,:));
    grads=zeros(numModel,num_cont);
    
    for k=1:numModel
        current=expandDesign(settings,model(:,:,k),ndif);
        objective(k) = -log(prod(svd(current)));
        gradient=zeros(1,num_cont);       
        currentSquared = current'*current;
        if rcond(currentSquared)>1e-6
            fixedProd = -2.*current(row,:)/currentSquared;
        else
            invCur = pinv(currentSquared); 
            fixedProd = -2.*current(row,:)*invCur;
        end
        
        for n=1:num_cont
            tmp = model(:,:,k);
            tmp(:,n) = tmp(:,n)-1;
            const = ones(length(model(:,n,k)),1);
            const(n) = model(n,n,k);
            derivative = const.*prod(settings(row,:).^tmp,2);
            derivative(model(:,n,k)<=0) = 0;
            count=1;
            for m=(length(model(:,n,k))-(2*ndif-1)):2:length(model(:,n,k))
                term1=model(m,:,k).*settings(row,:);
                term1=prod(term1(find(term1)));
                term2=settings(row,(model(m+1,:,k)==2))^2;
                derivative(end-ndif+count)=term1-term2;
                count=count+1;
            end
            gradient(n)=fixedProd*derivative;
        end
        grads(k,:)=gradient;
    end
    objective=mean(objective);
    gradient=(1/numModel).*sum(grads,1); 

end
